@@ -8,17 +8,9 @@ import warnings
 import numpy as np
 from tlz import concat, frequencies
 
-from ..highlevelgraph import HighLevelGraph
-from ..utils import has_keyword, is_arraylike, is_cupy_type
-from .core import Array
-
-try:
-    AxisError = np.AxisError
-except AttributeError:
-    try:
-        np.array([0]).sum(axis=5)
-    except Exception as e:
-        AxisError = type(e)
+from dask.array.core import Array
+from dask.highlevelgraph import HighLevelGraph
+from dask.utils import has_keyword, is_arraylike, is_cupy_type
 
 
 def normalize_to_array(x):
@@ -96,6 +88,8 @@ def meta_from_array(x, ndim=None, dtype=None):
                 meta = meta.sum()
             else:
                 meta = meta.reshape((0,) * ndim)
+        if meta is np.ma.masked:
+            meta = np.ma.array(np.empty((0,) * ndim, dtype=dtype or x.dtype), mask=True)
     except Exception:
         meta = np.empty((0,) * ndim, dtype=dtype or x.dtype)
 
@@ -179,7 +173,10 @@ def allclose(a, b, equal_nan=False, **kwargs):
     a = normalize_to_array(a)
     b = normalize_to_array(b)
     if getattr(a, "dtype", None) != "O":
-        return np.allclose(a, b, equal_nan=equal_nan, **kwargs)
+        if hasattr(a, "mask") or hasattr(b, "mask"):
+            return np.ma.allclose(a, b, masked_equal=True, **kwargs)
+        else:
+            return np.allclose(a, b, equal_nan=equal_nan, **kwargs)
     if equal_nan:
         return a.shape == b.shape and all(
             np.isnan(b) if np.isnan(a) else a == b for (a, b) in zip(a.flat, b.flat)
@@ -222,19 +219,25 @@ def assert_eq_shape(a, b, check_nan=True):
             assert aa == bb
 
 
-def _check_chunks(x):
-    x = x.persist(scheduler="sync")
+def _check_chunks(x, scheduler=None):
+    x = x.persist(scheduler=scheduler)
     for idx in itertools.product(*(range(len(c)) for c in x.chunks)):
         chunk = x.dask[(x.name,) + idx]
+        if hasattr(chunk, "result"):  # it's a future
+            chunk = chunk.result()
         if not hasattr(chunk, "dtype"):
             chunk = np.array(chunk, dtype="O")
         expected_shape = tuple(c[i] for c, i in zip(x.chunks, idx))
         assert_eq_shape(expected_shape, chunk.shape, check_nan=False)
-        assert chunk.dtype == x.dtype
+        assert (
+            chunk.dtype == x.dtype
+        ), "maybe you forgot to pass the scheduler to `assert_eq`?"
     return x
 
 
-def _get_dt_meta_computed(x, check_shape=True, check_graph=True, check_chunks=True):
+def _get_dt_meta_computed(
+    x, check_shape=True, check_graph=True, check_chunks=True, scheduler=None
+):
     x_original = x
     x_meta = None
     x_computed = None
@@ -247,8 +250,8 @@ def _get_dt_meta_computed(x, check_shape=True, check_graph=True, check_chunks=Tr
         x_meta = getattr(x, "_meta", None)
         if check_chunks:
             # Replace x with persisted version to avoid computing it twice.
-            x = _check_chunks(x)
-        x = x.compute(scheduler="sync")
+            x = _check_chunks(x, scheduler=scheduler)
+        x = x.compute(scheduler=scheduler)
         x_computed = x
         if hasattr(x, "todense"):
             x = x.todense()
@@ -274,6 +277,9 @@ def assert_eq(
     check_meta=True,
     check_chunks=True,
     check_type=True,
+    check_dtype=True,
+    equal_nan=True,
+    scheduler="sync",
     **kwargs,
 ):
     a_original = a
@@ -285,13 +291,21 @@ def assert_eq(
         b = np.array(b)
 
     a, adt, a_meta, a_computed = _get_dt_meta_computed(
-        a, check_shape=check_shape, check_graph=check_graph, check_chunks=check_chunks
+        a,
+        check_shape=check_shape,
+        check_graph=check_graph,
+        check_chunks=check_chunks,
+        scheduler=scheduler,
     )
     b, bdt, b_meta, b_computed = _get_dt_meta_computed(
-        b, check_shape=check_shape, check_graph=check_graph, check_chunks=check_chunks
+        b,
+        check_shape=check_shape,
+        check_graph=check_graph,
+        check_chunks=check_chunks,
+        scheduler=scheduler,
     )
 
-    if str(adt) != str(bdt):
+    if check_dtype and str(adt) != str(bdt):
         raise AssertionError(f"a and b have different dtypes: (a: {adt}, b: {bdt})")
 
     try:
@@ -344,7 +358,7 @@ def assert_eq(
                         )
                         assert type(b_meta) == type(b_computed), msg
         msg = "found values in 'a' and 'b' which differ by more than the allowed amount"
-        assert allclose(a, b, **kwargs), msg
+        assert allclose(a, b, equal_nan=equal_nan, **kwargs), msg
         return True
     except TypeError:
         pass
@@ -422,7 +436,7 @@ def array_safe(a, like, **kwargs):
     to convert a `dask.Array` and CuPy doesn't implement `__array__` to
     prevent implicit copies to host.
     """
-    from .routines import array
+    from dask.array.routines import array
 
     return _array_like_safe(np.array, array, a, like, **kwargs)
 
@@ -436,7 +450,7 @@ def asarray_safe(a, like, **kwargs):
     a.compute(scheduler="sync") before np.asarray, as downstream
     libraries are unlikely to know how to convert a dask.Array.
     """
-    from .core import asarray
+    from dask.array.core import asarray
 
     return _array_like_safe(np.asarray, asarray, a, like, **kwargs)
 
@@ -450,7 +464,7 @@ def asanyarray_safe(a, like, **kwargs):
     a.compute(scheduler="sync") before np.asanyarray, as downstream
     libraries are unlikely to know how to convert a dask.Array.
     """
-    from .core import asanyarray
+    from dask.array.core import asanyarray
 
     return _array_like_safe(np.asanyarray, asanyarray, a, like, **kwargs)
 
@@ -462,7 +476,7 @@ def validate_axis(axis, ndim):
     if not isinstance(axis, numbers.Integral):
         raise TypeError("Axis value must be an integer, got %s" % axis)
     if axis < -ndim or axis >= ndim:
-        raise AxisError(
+        raise np.AxisError(
             "Axis %d is out of bounds for array of dimension %d" % (axis, ndim)
         )
     if axis < 0:
@@ -530,3 +544,17 @@ def scipy_linalg_safe(func_name, *args, **kwargs):
 
 def solve_triangular_safe(a, b, lower=False):
     return scipy_linalg_safe("solve_triangular", a, b, lower=lower)
+
+
+def __getattr__(name):
+    # Can't use the @_deprecated decorator as it would not work on `except AxisError`
+    if name == "AxisError":
+        warnings.warn(
+            "AxisError was deprecated after version 2021.10.0 and will be removed in a "
+            "future release. Please use numpy.AxisError instead.",
+            category=FutureWarning,
+            stacklevel=2,
+        )
+        return np.AxisError
+    else:
+        raise AttributeError(f"module {__name__} has no attribute {name}")
