@@ -3,8 +3,8 @@ from collections.abc import Mapping
 from io import BytesIO
 from warnings import catch_warnings, simplefilter, warn
 
-from ...highlevelgraph import HighLevelGraph
-from ...layers import DataFrameIOLayer
+from dask.highlevelgraph import HighLevelGraph
+from dask.layers import DataFrameIOLayer
 
 try:
     import psutil
@@ -27,13 +27,13 @@ from pandas.api.types import (
     is_object_dtype,
 )
 
-from ...base import tokenize
-from ...bytes import read_bytes
-from ...core import flatten
-from ...delayed import delayed
-from ...utils import asciitable, parse_bytes
-from ..core import new_dd_object
-from ..utils import clear_known_categories
+from dask.base import tokenize
+from dask.bytes import read_bytes
+from dask.core import flatten
+from dask.dataframe.core import new_dd_object
+from dask.dataframe.utils import clear_known_categories
+from dask.delayed import delayed
+from dask.utils import asciitable, parse_bytes
 
 
 class CSVFunctionWrapper:
@@ -517,6 +517,10 @@ def read_pandas(
             2
         ]
 
+        # Check for at least one valid path
+        if len(paths) == 0:
+            raise OSError(f"{urlpath} resolved to no files")
+
         # Infer compression from first path
         compression = infer_compression(paths[0])
 
@@ -959,6 +963,6 @@ def to_csv(
         return values
 
 
-from ..core import _Frame
+from dask.dataframe.core import _Frame
 
 _Frame.to_csv.__doc__ = to_csv.__doc__
