@@ -4,10 +4,10 @@ import numpy as np
 import pandas as pd
 from tlz import partition
 
-from ._compat import PANDAS_GT_131
+from dask.dataframe._compat import PANDAS_GT_131
 
 #  preserve compatibility while moving dispatch objects
-from .dispatch import (  # noqa: F401
+from dask.dataframe.dispatch import (  # noqa: F401
     concat,
     concat_dispatch,
     group_split_dispatch,
@@ -18,7 +18,7 @@ from .dispatch import (  # noqa: F401
     tolist_dispatch,
     union_categoricals,
 )
-from .utils import is_dataframe_like, is_index_like, is_series_like
+from dask.dataframe.utils import is_dataframe_like, is_index_like, is_series_like
 
 # cuDF may try to import old dispatch functions
 hash_df = hash_object_dispatch
@@ -402,6 +402,14 @@ def pivot_agg(df):
     return df.groupby(level=0).sum()
 
 
+def pivot_agg_first(df):
+    return df.groupby(level=0).first()
+
+
+def pivot_agg_last(df):
+    return df.groupby(level=0).last()
+
+
 def pivot_sum(df, index, columns, values):
     return pd.pivot_table(
         df, index=index, columns=columns, values=values, aggfunc="sum", dropna=False
@@ -416,6 +424,18 @@ def pivot_count(df, index, columns, values):
     ).astype(np.float64)
 
 
+def pivot_first(df, index, columns, values):
+    return pd.pivot_table(
+        df, index=index, columns=columns, values=values, aggfunc="first", dropna=False
+    )
+
+
+def pivot_last(df, index, columns, values):
+    return pd.pivot_table(
+        df, index=index, columns=columns, values=values, aggfunc="last", dropna=False
+    )
+
+
 def assign_index(df, ind):
     df = df.copy()
     df.index = ind
@@ -423,11 +443,13 @@ def assign_index(df, ind):
 
 
 def monotonic_increasing_chunk(x):
-    data = x if is_index_like(x) else x.iloc
-    return pd.DataFrame(
-        data=[[x.is_monotonic_increasing, data[0], data[-1]]],
-        columns=["monotonic", "first", "last"],
-    )
+    if x.empty:
+        # if input is empty, return empty df for chunk
+        data = None
+    else:
+        data = x if is_index_like(x) else x.iloc
+        data = [[x.is_monotonic_increasing, data[0], data[-1]]]
+    return pd.DataFrame(data=data, columns=["monotonic", "first", "last"])
 
 
 def monotonic_increasing_aggregate(concatenated):
@@ -438,11 +460,13 @@ def monotonic_increasing_aggregate(concatenated):
 
 
 def monotonic_decreasing_chunk(x):
-    data = x if is_index_like(x) else x.iloc
-    return pd.DataFrame(
-        data=[[x.is_monotonic_decreasing, data[0], data[-1]]],
-        columns=["monotonic", "first", "last"],
-    )
+    if x.empty:
+        # if input is empty, return empty df for chunk
+        data = None
+    else:
+        data = x if is_index_like(x) else x.iloc
+        data = [[x.is_monotonic_decreasing, data[0], data[-1]]]
+    return pd.DataFrame(data=data, columns=["monotonic", "first", "last"])
 
 
 def monotonic_decreasing_aggregate(concatenated):
