@@ -18,19 +18,19 @@ from dask.utils import (
     cached_cumsum,
     derived_from,
     ensure_dict,
+    ensure_set,
     extra_titles,
+    factors,
     format_bytes,
     funcname,
     getargspec,
     has_keyword,
-    ignoring,
     is_arraylike,
     itemgetter,
     iter_chunks,
     memory_repr,
     methodcaller,
     ndeepmap,
-    noop_context,
     parse_bytes,
     parse_timedelta,
     partial_by_order,
@@ -220,7 +220,7 @@ def test_random_state_data():
 
 def test_memory_repr():
     for power, mem_repr in enumerate(["1.0 bytes", "1.0 KB", "1.0 MB", "1.0 GB"]):
-        assert memory_repr(1024 ** power) == mem_repr
+        assert memory_repr(1024**power) == mem_repr
 
 
 def test_method_caller():
@@ -468,6 +468,22 @@ def test_ensure_dict():
         assert di == d
 
 
+def test_ensure_set():
+    s = {1}
+    assert ensure_set(s) is s
+
+    class myset(set):
+        pass
+
+    s2 = ensure_set(s, copy=True)
+    s3 = ensure_set(myset(s))
+
+    for si in (s2, s3):
+        assert type(si) is set
+        assert si is not s
+        assert si == s
+
+
 def test_itemgetter():
     data = [1, 2, 3]
     g = itemgetter(1)
@@ -569,7 +585,7 @@ def test_parse_bytes():
     assert parse_bytes("5kB") == 5000
     assert parse_bytes("5.4 kB") == 5400
     assert parse_bytes("1kiB") == 1024
-    assert parse_bytes("1Mi") == 2 ** 20
+    assert parse_bytes("1Mi") == 2**20
     assert parse_bytes("1e6") == 1000000
     assert parse_bytes("1e6 kB") == 1000000000
     assert parse_bytes("MB") == 1000000
@@ -604,6 +620,14 @@ def test_parse_timedelta():
     assert parse_timedelta("1", default="ms") == 0.001
     assert parse_timedelta(1, default="ms") == 0.001
 
+    assert parse_timedelta("1ms", default=False) == 0.001
+    with pytest.raises(ValueError):
+        parse_timedelta(1, default=False)
+    with pytest.raises(ValueError):
+        parse_timedelta("1", default=False)
+    with pytest.raises(TypeError):
+        parse_timedelta("1", default=None)
+
 
 def test_is_arraylike():
     np = pytest.importorskip("numpy")
@@ -689,15 +713,15 @@ def test_stringify_collection_keys():
         (0, "0 B"),
         (920, "920 B"),
         (930, "0.91 kiB"),
-        (921.23 * 2 ** 10, "921.23 kiB"),
-        (931.23 * 2 ** 10, "0.91 MiB"),
-        (921.23 * 2 ** 20, "921.23 MiB"),
-        (931.23 * 2 ** 20, "0.91 GiB"),
-        (921.23 * 2 ** 30, "921.23 GiB"),
-        (931.23 * 2 ** 30, "0.91 TiB"),
-        (921.23 * 2 ** 40, "921.23 TiB"),
-        (931.23 * 2 ** 40, "0.91 PiB"),
-        (2 ** 60, "1024.00 PiB"),
+        (921.23 * 2**10, "921.23 kiB"),
+        (931.23 * 2**10, "0.91 MiB"),
+        (921.23 * 2**20, "921.23 MiB"),
+        (931.23 * 2**20, "0.91 GiB"),
+        (921.23 * 2**30, "921.23 GiB"),
+        (931.23 * 2**30, "0.91 TiB"),
+        (921.23 * 2**40, "921.23 TiB"),
+        (931.23 * 2**40, "0.91 PiB"),
+        (2**60, "1024.00 PiB"),
     ],
 )
 def test_format_bytes(n, expect):
@@ -757,18 +781,6 @@ def test_deprecated_message():
     assert str(record[0].message) == "woohoo"
 
 
-def test_ignoring_deprecated():
-    with pytest.warns(FutureWarning, match="contextlib.suppress"):
-        with ignoring(ValueError):
-            pass
-
-
-def test_noop_context_deprecated():
-    with pytest.warns(FutureWarning, match="contextlib.nullcontext"):
-        with noop_context():
-            pass
-
-
 def test_typename():
     assert typename(HighLevelGraph) == "dask.highlevelgraph.HighLevelGraph"
     assert typename(HighLevelGraph, short=True) == "dask.HighLevelGraph"
@@ -805,3 +817,11 @@ def test_cached_cumsum_non_tuple():
     assert cached_cumsum(a) == (1, 3, 6)
     a[1] = 4
     assert cached_cumsum(a) == (1, 5, 8)
+
+
+def test_factors():
+    assert factors(0) == set()
+    assert factors(1) == {1}
+    assert factors(2) == {1, 2}
+    assert factors(12) == {1, 2, 3, 4, 6, 12}
+    assert factors(15) == {1, 3, 5, 15}
